import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
/* eslint-disable sonarjs/sonar-no-unused-class-component-methods */
/* eslint-disable sonarjs/no-unsafe */
/* eslint-disable sonarjs/public-static-readonly */
/* eslint-disable @typescript-eslint/ban-types */
import { Component } from 'react';
const DefaultLoader = () => null;
const loadingStates = new WeakMap();
const getOrCreateLoadingState = (loadFn) => {
    if (!loadingStates.has(loadFn)) {
        loadingStates.set(loadFn, {
            error: null,
            loading: true,
        });
    }
    return loadingStates.get(loadFn);
};
const updateLoadingState = (loadFn, update) => {
    const loadingState = getOrCreateLoadingState(loadFn);
    const updatedState = { ...loadingState, ...update };
    loadingStates.set(loadFn, updatedState);
    return updatedState;
};
const load = (loadFn) => {
    const state = getOrCreateLoadingState(loadFn);
    if (state.result) {
        return state;
    }
    updateLoadingState(loadFn, { loading: true, error: null });
    state.loading = true;
    state.error = null;
    state.promise = loadFn()
        .then((result) => {
        state.loading = false;
        state.result = result;
        updateLoadingState(loadFn, state);
        return result;
    })
        .catch((error) => {
        state.loading = false;
        state.error = error;
        updateLoadingState(loadFn, state);
        throw error;
    });
    updateLoadingState(loadFn, state);
    return state;
};
class LoadableComponentBase extends Component {
    config;
    constructor(config, initialState, props) {
        super(props);
        this.config = config;
        this.state = initialState;
    }
    static loadFn = undefined;
    static preload() {
        load(this.loadFn);
    }
    UNSAFE_componentWillMount() {
        this.loadModule();
    }
    loadModule() {
        if (!this.state.loading) {
            return;
        }
        const { promise } = load(this.config.loadFn);
        this.syncLoadingState();
        promise
            .then(() => {
            this.syncLoadingState();
            return null;
        })
            .catch(() => {
            this.syncLoadingState();
            return null;
        });
    }
    syncLoadingState() {
        this.setState(getOrCreateLoadingState(this.config.loadFn));
    }
    updateLoadingState(update) {
        this.setState(updateLoadingState(this.config.loadFn, update));
    }
    render() {
        const { loader } = this.config;
        const Loader = loader || DefaultLoader;
        if (this.state.loading || this.state.error) {
            return _jsx(Loader, {});
        }
        else if (this.state.result) {
            const Component = this.state.result;
            const Extra = this.config.extra;
            return (_jsxs(_Fragment, { children: [_jsx(Component, { ...this.props }), Extra && _jsx(Extra, {})] }));
        }
        else {
            return null;
        }
    }
}
/* eslint-disable-next-line @typescript-eslint/no-explicit-any */
export function loadable(loadFn, loaderOrConfig) {
    const config = typeof loaderOrConfig === 'object'
        ? loaderOrConfig
        : { loader: loaderOrConfig };
    class LoadableComponent extends LoadableComponentBase {
        loadFn = loadFn;
        constructor(props) {
            super({
                loadFn,
                ...config,
            }, getOrCreateLoadingState(loadFn), props);
        }
    }
    return LoadableComponent;
}
